% Nom du fichier : construction_matrice_rigidite.m
% 
% Appel de la fonction :
% [Kg,E,Nu]=construction_matrice_rigidite(E,Nu,ne,Tc,l,h);
% 
% DESCRIPTION GENERALE
% - Fonction permettant d'assembler la martice de rigidit
% - Dformations planes ou contraintes planes ( rgler dans Kelem)
% - Mthode des lments finis
% - Elments utiliss type plane42 d'ANSYS (fonctions d'interpolation
% linaire)
% 
% DESCRITPION DES ENTREES DE LA FONCTION
% - E, rigidite = matrice n*n pixels donnant la repartition rigidite du
%   substrat. "rigidite(20,35)" doit correspondre au pixel de coordonnes (20,35)
% - Nu, coeff de Poisson = Constante, compressibilit du matriau. -1.0<Nu<0.5
% (Nu = 0.5 ou Nu = -1 => s'accompagnent d'instabilits numriques (/0))
% 
% DESCRIPTION DES SORTIES DE LA FONCTION
% - Kg = matrice de rigidit assemble mais non rorganise (non prise en
% compte des conditions limites)
% 
%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm


function [Kg,E,Nu]=construction_matrice_rigidite(E,Nu,ne,Tc,l,h)

% disp(' ')
% disp(' Traitement du pb direct :')
% disp('-----------------------------------------------------')
% t11=cputime;

%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
% Donnes maillage :

% Nbre de noeuds sur l (X ou direction 1) :
nn(1)=ne(1)+1;
% Taille des lments suivant X (direction 1) :
te(1)=l/ne(1);
% Nbre de noeuds sur h (Y ou direction 2) :
nn(2)=ne(2)+1;
% Taille des lments suivant Y (direction 2) :
te(2)=h/ne(2);
% Nbre de noeuds au total :
nnt=prod(nn);
% Nbre d'elemnts :
net=prod(ne);
%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm


%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
% Definition matriau (linaire isotrope) :
%
% Cartographie du module d'Young de chaque lment :
% Module d'Young (kPa rester coherent):
E = reshape(E,ne(1)*ne(2),1);
%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm


%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
% Assemblage :
% disp('        -> Assemblage de Kg');
%
% Les matrices de rigidits sont donnes par la fonction
% K_elem(E,nu,a_elem,b_elem)
%
% L'assemblage se droule tel que le vecteur U soit de la forme :
%  Ux1
%  Uy1
%  Ux2
%  Uy2
%  .
%[ .  ]
%  .
%  UxN
%  UyN
%
% On utilise pour cela la table de connectivit donne plus haut.
% Pour chaque lment, on peut crer une matrice Ke, dont la taille est
% celle de Kg, et dont les membres sont ceux donns par la fonction
% K_elem.
%Kg=sparse(nnt*2,nnt*2); % Time consuming
Kg=spalloc(nnt*2,nnt*2,13*2*nnt-round(nnt/5));
%K_nbre_Mat_Elem=Kg;
iii = 0;
i = zeros(1,4*4*4*net);
j = i;
s = i;
for ii=1:1:net
    %disp(['ii/net*100 = ' num2str(ii/net*100)])
    
    % Appel de la function K_elem pour avoir la matrice de rigidite
    % elementaire. Ke est fonction de l'lment. Chaque lment peut avoir
    % son propre module E dans le tableau E. Ici l'intrpolation a t faite
    %  la main, donc il n'est pas la partie interpolation numerique sur
    % chaque lment.
    alpha1 = 1;
    Ke=K_elem(E(ii),Nu,te(1),te(2),alpha1);
    
    % Assemblage
    % Mise en position des termes de la matrice lementaire ds Ke_g :
    for jj=1:1:4
        % Indice donnant la position dans la matrice globale suivant les
        % lignes
        nnl=Tc(ii,jj); % numero_noeud_ligne
        for kk=1:1:4
            
            % Indice donnant la position dans la matrice globale suivant les
            % colonnes
            nnc=Tc(ii,kk); %numero_noeud_colonne
            
%             % Construction naive et lente (vous pouvez essayer avec
%             % ne<100)...
%             % Lignes concernant les Ux :
%             Kg(nnl*2-1,nnc*2-1)=Kg(nnl*2-1,nnc*2-1)+Ke(2*jj-1,2*kk-1);
%             Kg(nnl*2-1,nnc*2)=Kg(nnl*2-1,nnc*2)+Ke(2*jj-1,2*kk);
%             
%             % Lignes concernant les Uy :
%             Kg(nnl*2,nnc*2-1)=Kg(nnl*2,nnc*2-1)+Ke(2*jj,2*kk-1);
%             Kg(nnl*2,nnc*2)=Kg(nnl*2,nnc*2)+Ke(2*jj,2*kk);
            
            % Construction moins naive :
            % Vecteurs d'assemblage :
            % Lignes concernant les Ux :
            iii=iii+1;
            i(iii)=nnl*2-1;
            j(iii)=nnc*2-1;
            s(iii)=Ke(2*jj-1,2*kk-1);
            iii=iii+1;
            i(iii)=nnl*2-1;
            j(iii)=nnc*2;
            s(iii)=Ke(2*jj-1,2*kk);
            
            % Lignes concernant les Uy :
            iii=iii+1;
            i(iii)=nnl*2;
            j(iii)=nnc*2-1;
            s(iii)=Ke(2*jj,2*kk-1);
            iii=iii+1;
            i(iii)=nnl*2;
            j(iii)=nnc*2;
            s(iii)=Ke(2*jj,2*kk);

        end
    end
end

% Assemblage construction moins naive:
Kg = sparse(i,j,s);

% Gain de place :
clear ii jj kk Ke nnl nnc

end
%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm



