% Nom du fichier : maillage_rectangle.m
% 
% Appel de la fonction :
% 
% DESCRIPTION GENERALE
% 
% DESCRITPION DES ENTREES DE LA FONCTION
%   - l : largeur du domaine
%   - h : hauteur du domaine
%   - ne : nbre d'lments [en hauteur, en largeur]
% DESCRIPTION DES SORTIES DE LA FONCTION
%   - Tnn : table des numros des noeuds 
%   - Tne : table des numros des lments
%   - Tc  : table de connectivit entre les lments et les noeuds
%   - te  : taille physique des lments
%   - X   : tableau de la position en X des noeuds
%   - Y   : tableau de la position en Y des noeuds
% EXEMPLE MATLAB D APPEL DE LA FONCTION
%
% Simon mars 2008
%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm

function [Tnn,Tne,Tc,te,X,Y]=maillage_rectangle(l,h,ne)

%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
% Donnes maillage :

% Nbre de noeuds sur l (X ou direction 1) :
nn(1)=ne(1)+1;
% Taille des lments suivant X (direction 1) :
te(1)=l/ne(1);
% Nbre de noeuds sur h (Y ou direction 2) :
nn(2)=ne(2)+1;
% Taille des lments suivant Y (direction 2) :
te(2)=h/ne(2);
% Nbre de noeuds au total :
%nnt=prod(nn);
% Nbre d'elemnts :
net=prod(ne);
%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm



%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
% Tables des lments et des noeuds :
% disp(' Mise en place du systme K.U = F :');
% disp('        -> Construction des tables des noeuds et des lments');
%
% Permettent de construire la table de connectivit.
% La programation des tables des numros des noeuds et des numros des elements est propre 
% la numrotation adopte :
%
%            %...
%              %...
%            %....
% noeud      %11 12  13  14  15  16  17  18  19  20
% elem         %1  %2  %3  %4  %5  %6  %7  %8  %9
% noeud      %1   2   3   4   5   6   7  8   9   10

% Table des numros des noeuds :
Tnn=zeros(nn(2),nn(1));
for ii=1:1:nn(2)
    for jj=1:1:nn(1)
        Tnn(ii,jj)=ii+nn(1)*(jj-1); %jj+nn(1)*(ii-1)
    end
end
clear ii jj

% Table des numros des lments :
% Tne=zeros(ne(2),ne(1));
% for ii=1:1:ne(2)
%     for jj=1:1:ne(1)
%         Tne(ii,jj)=ii+ne(1)*(jj-1); %jj+ne(1)*(ii-1)
%     end
% end
Tne=zeros(ne(2),ne(1));
for ii=1:1:ne(2)
    for jj=1:1:ne(1)
        Tne(ii,jj)=ii+ne(2)*(jj-1); %jj+ne(1)*(ii-1)
    end
end
clear ii jj
%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm

%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
% Table de connectivit :
% disp('        -> Construction de la table de connectivit');
%
% La table de connectivit donne, pour chaque numro lment repr par son numro,
% les 4 numros des noeuds attach  cet lment. Par exemple,
% Tc(14,:) donne les numros des quatre noeuds attach  cet lment
% (par exemple Tc(1,4) donnera 12, le 4ieme noeud du 1ier lment).
%
% Entres :
% Tne donne les numros des lments rpartis tel que les
% elements joints le soient aussi ds cette table. La liste des numros des
% noeuds doit etre continue, tel que max(Tne) soit rellement
% le nbre d'lements.
% Tnn donne les numros des noeuds en correspondance 
% l'ordre dfini des lments dans la table prcdente.
%
% Note : Cette fonction ne marche que pour des lments rectangulaires (pas
% de bords autres que verticaux ou horizontaux car a n'a pas t test).


% Pour la table de connectivite :
% noeud 4 %%%%%%%%%%%%%%%%% noeud 3
%                                 %
%                                 %
%                                 %
%                                 % longueur b
%                                 %
%                                 %
% noeud 1 %%%%%%%%%%%%%%%%% noeud 2
%            longueur a


Tc=zeros(net,4);
onesss = ones(ne(2),ne(1));
for numero_elem=1:1:net
    % On trouve les indices ii et jj dans la matrice Tne
    % correspondant  numero_elem
    [ii,jj]=find((numero_elem*onesss-Tne)==0);
    
    % On remplit ainsi aisment la table de connectivit avec n'importe
    % quel table des numros d'lments et tables des numoros des noeuds
    % correspondante.
    Tc(numero_elem,1)=Tnn(ii,jj);
    Tc(numero_elem,2)=Tnn(ii,jj+1);
    Tc(numero_elem,4)=Tnn(ii+1,jj);
    Tc(numero_elem,3)=Tnn(ii+1,jj+1);
end
clear ii jj numero_elem


% Dans un tableau qui represente les noeuds on met la valeur des positions
% de ces derniers :
X=0:te(1):l;
Y=0:te(2):h;
[X,Y]=meshgrid(X,Y);

%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm

end