% Nom du fichier : resolution.m
% 
% Appel de la fonction :
% [Ux,Uy,Fx,Fy]=resolution(Kg,U,F);
% 
% DESCRIPTION GENERALE
% - Partitionnement du problme linaire
% - Rsolution squentielle 1) du pb en dplacement 2) du pb en force
% 
% DESCRITPION DES ENTREES DE LA FONCTION
% - Kg matrice de rigidit du systme total.
% - U vecteur dplacement des noeuds. alternance composante x,y. "i" veut
% dire force impose
% - F vecteur force impose aux noeuds. alternance composante x,y. "i" veut
% dire dplacement impos
%
% DESCRIPTION DES SORTIES DE LA FONCTION
% - Ux,Uy : composantes du vecteur dplacement des noeuds
% - Fx,Fy : composantes du vecteur force extrieures aux noeuds
% 
%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm


function [U,F,Ux,Uy,Fx,Fy]=resolution(Kg,U,F)


%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
% Rorganisation des matrices (ou partitionnement)
% disp('        -> Rorganisation de Kg');
% t1=cputime;

nnt = size(F,1)/2; % Nombre total de noeuds

% Nous construisons ainsi 4 matrices.
% K1 est la matrice des dplacements inconnus telles que les forces soient
% connues.
% K2 est la matrice des dplacements connus telles que les forces
% correspondantes soient connues. K2 insparable de K1 au niveau quation
% du sys linaire.
% K3 est la matrice des dplacements initialement inconnus telles que les forces
% soient inconnues.
% K4 est la matrice des dplacements initialement connus telles que les forces
% soient connues. K4 insparable de K3 au niveau quation
% du sys linaire.

% Stockage de la matrice de rigidit :
%K_stock=Kg;
%save 'K_stock.mat' K_stock
%clear K_stock

% Recherche des indices des colonnes et lignes de la matrice Kg correspondant
%  K2 et K4
indu=0;
indf=0;
for ii=1:1:2*nnt
    if F(ii)==i
        indf=indf+1;
        Liste_inconnues_F(indf)=ii;
    end
end
clear indu indf
% Liste_connus_U=Liste_inconnues_F: Si ce n'est pas le cas, il y a un pb !!

% Construction du vecteur qui reorganise la matrice Kg :
reorga=zeros(1,2*nnt);
ind=1;
count=1;
for ii=1:1:2*nnt
    if ii==Liste_inconnues_F(ind)
        if ind < max(size(Liste_inconnues_F))
            ind=ind+1;
        end
    else
        reorga(count)=ii;
        count=count+1;
    end
end

ind=1;
for ii=2*nnt-max(size(Liste_inconnues_F))+1:1:2*nnt
    reorga(ii)=Liste_inconnues_F(ind);
    if ind < max(size(Liste_inconnues_F))
            ind=ind+1;
    end
end
clear ii ind count

% Reorganisation de la matrice Kg grace  ce vecteur :
% Premiere etape : les colonnes (correspond aux deplacements U) :
Kg=Kg(:,reorga);
% Deuxieme etape : les lignes (correspond aux forces F) :
Kg=Kg(reorga,:);

% Cration des matrices K1, K2 K3 et K4  partir de Kg reorganisee:
K1=Kg(  1 : 2*nnt-max(size(Liste_inconnues_F))  ,  1 : 2*nnt-max(size(Liste_inconnues_F))  );
K2=Kg(  1:2*nnt-max(size(Liste_inconnues_F))  ,  2*nnt-max(size(Liste_inconnues_F))+1 : 2*nnt  );
K3=Kg(  2*nnt-max(size(Liste_inconnues_F))+1 : 2*nnt  ,  1:2*nnt-max(size(Liste_inconnues_F))  );
K4=Kg(  2*nnt-max(size(Liste_inconnues_F))+1 : 2*nnt  ,  2*nnt-max(size(Liste_inconnues_F))+1 : 2*nnt  );

% On a plus besoin de Kg, on l'efface !
clear Kg

% Reorganisation de F et U:
F=F(reorga,:);
U=U(reorga,:);
% Vecteur Uc (deplacements imposs):
Uc=U(2*nnt-max(size(Liste_inconnues_F))+1 : 2*nnt , 1);
% Vecteur Fc (forces imposes ou nulle):
Fc=F( 1 : 2*nnt-max(size(Liste_inconnues_F)) , 1);

% temps pass  rorganiser K :
% t2=cputime;
% disp(['        -> Dure de la rorganisation : ' num2str(t2-t1) ' s']);

%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm




%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
% Resolution du pb :
% disp(' Rsolution du pb linaire :');

% Rsolution du systme linaire :
% Algorithme de rsolution utilis ? L'aide matlab indique, pour notre cas:
% - Matrice sparse, symmetric, real positive diagonal => factorisation cholesky
disp(['Choix de l''algorithme de rsolution du systme linaire'])
spparms('spumoni',1); % Gets info on which algorithm used.
Ui=K1\(Fc-K2*Uc); % Resolution


% Deuxieme tape : Trouver les Fi:
Fi=K3*Ui+K4*Uc;
% t2=cputime;
% disp(['        -> Dure des multiplications matricielles : ' num2str(t2-t1) ' s']);
% clear t1 t2

% Reconstruction de U et de F:
U( 1 : 2*nnt-max(size(Liste_inconnues_F)) , 1 ) = Ui;
F(2*nnt-max(size(Liste_inconnues_F))+1 : 2*nnt , 1 )= Fi;

% Reorganisation des vecteurs comme ils taient avant la rorganisation des matrices pour la rsolution :
inv_reorga=zeros(1,2*nnt);
for ii=1:1:2*nnt
    inv_reorga(reorga(ii))=ii;
end
F=F(inv_reorga,:);
U=U(inv_reorga,:);

% Classement des Ux, Uy Fx et Fy  partir de U et F :
for ii=1:1:nnt
    Ux(ii)=U(2*ii-1);
    Uy(ii)=U(2*ii);
    Fx(ii)=F(2*ii-1);
    Fy(ii)=F(2*ii);
end
clear ii

clear Liste_inconnues_F K1 K2 K3 K4 inv_reorga reorga Ui Fi Uc Fc
%mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm


end % Fin de la function
